//-----------------------------------------------------------------------------
// Copyright (c) 2007 dhpoware. All Rights Reserved.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
// IN THE SOFTWARE.
//-----------------------------------------------------------------------------

#include <iostream>
#include "thread.h"

Event g_threadFinished;

void ThreadFunc()
{
    std::cout << "Hello, world!" << std::endl;
    g_threadFinished.notify();
}

const char *ThreadPriorityToStr(Thread::Priority priority)
{
    switch (priority)
    {
    case Thread::PRIORITY_LOWEST:
        return "PRIORITY_LOWEST";

    case Thread::PRIORITY_BELOW_NORMAL:
        return "PRIORITY_BELOW_NORMAL";

    case Thread::PRIORITY_NORMAL:
        return "PRIORITY_NORMAL";

    case Thread::PRIORITY_ABOVE_NORMAL:
        return "PRIORITY_ABOVE_NORMAL";

    case Thread::PRIORITY_HIGHEST:
        return "PRIORITY_HIGHEST";

    default:
        return "UNKNOWN PRIORITY";
    }
}

int main()
{
    Thread t;

    if (t.create("My Thread"))
    {
        std::cout << t.name() << " created" << std::endl;

        std::cout << "Thread's current priority is: "
                  << ThreadPriorityToStr(t.getPriority()) << std::endl;

        std::cout << "Increasing priority to PRIORITY_HIGHEST" << std::endl;
        t.setPriority(Thread::PRIORITY_HIGHEST);
        
        std::cout << "Thread's current priority is: "
            << ThreadPriorityToStr(t.getPriority()) << std::endl;

        t.start(ThreadFunc);
        
        g_threadFinished.wait();
        std::cout << t.name() << " signals that it's work is done!" << std::endl;

        //t.join();
    }

    std::cout << "Press enter to exit";
    std::cin.get();

    return 0;
}